# -*- coding: utf-8 -*-
#
# File: googlecoop.py
#
# Copyright (c) 2007 by []
# Generator: ArchGenXML Version 1.6.0-beta-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Christian Ledermann <unknown>"""
__docformat__ = 'plaintext'


# There are three ways to inject custom code here:
#
#   - To set global configuration variables, create a file AppConfig.py.
#       This will be imported in config.py, which in turn is imported in
#       each generated class and in this file.
#   - To perform custom initialisation after types have been registered,
#       use the protected code section at the bottom of initialize().

import logging
logger = logging.getLogger('googlecoop')
logger.debug('Installing Product')

import Products.CMFPlone.interfaces
import os
import os.path
from Globals import package_home
from Products.Archetypes import listTypes
from Products.Archetypes.atapi import *
from Products.Archetypes.utils import capitalize
from Products.CMFCore import DirectoryView
from Products.CMFCore import permissions as cmfpermissions
from Products.CMFCore import utils as cmfutils
from Products.CMFPlone.utils import ToolInit
from Products.GenericSetup import EXTENSION
from Products.GenericSetup import profile_registry
from config import *

DirectoryView.registerDirectory('skins', product_globals)
DirectoryView.registerDirectory('skins/googlecoop',
                                    product_globals)

##code-section custom-init-head #fill in your manual code here
##/code-section custom-init-head


def initialize(context):
    ##code-section custom-init-top #fill in your manual code here
    ##/code-section custom-init-top

    # imports packages and types for registration

    import googlecoop

    # Initialize portal tools
    tools = [googlecoop.googlecoop]
    ToolInit( PROJECTNAME +' Tools',
                tools = tools,
                icon='tool.gif'
                ).initialize( context )

    # Initialize portal content
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    cmfutils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = DEFAULT_ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)

    profile_registry.registerProfile(
        name='default',
        title=PROJECTNAME,
        description='Profile for googlecoop',
        path='profiles/default',
        product='googlecoop',
        profile_type=EXTENSION,
        for_=Products.CMFPlone.interfaces.IPloneSiteRoot)

    ##code-section custom-init-bottom #fill in your manual code here
    ##/code-section custom-init-bottom




import unittest

#from zope.testing import doctestunit
#from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
ptc.setupPloneSite()

import Products.googlecoop


class TestCase(ptc.PloneTestCase):

    class layer(PloneSite):

        @classmethod
        def setUp(cls):
            fiveconfigure.debug_mode = True
            ztc.installPackage(Products.googlecoop)
            fiveconfigure.debug_mode = False

        @classmethod
        def tearDown(cls):
            pass


def test_suite():
    return unittest.TestSuite([

        # Unit tests
        #doctestunit.DocFileSuite(
        #    'README.txt', package='Products.googlecoop',
        #    setUp=testing.setUp, tearDown=testing.tearDown),

        #doctestunit.DocTestSuite(
        #    module='Products.googlecoop.mymodule',
        #    setUp=testing.setUp, tearDown=testing.tearDown),


        # Integration tests that use PloneTestCase
        #ztc.ZopeDocFileSuite(
        #    'README.txt', package='Products.googlecoop',
        #    test_class=TestCase),

        #ztc.FunctionalDocFileSuite(
        #    'browser.txt', package='Products.googlecoop',
        #    test_class=TestCase),

        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
#
# File: googlecoop.py
#
# Copyright (c) 2007 by []
# Generator: ArchGenXML Version 1.6.0-beta-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Christian Ledermann <unknown>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.googlecoop.config import *


from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
import md5
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr
##/code-section module-header

schema = Schema((

    StringField(
        name='identifier',
        default="009744842749537478185:hwbuiarvsbo",
        widget=StringWidget(
            description="Your Custom Search Engine's unique identifier",
            label="Identifier",
            label_msgid='googlecoop_label_identifier',
            description_msgid='googlecoop_help_identifier',
            i18n_domain='googlecoop',
        ),
        required=True,
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='adsposition',
        widget=SelectionWidget(
            label="Show ads on results pages",
            description="Specify where in the results you want advertising to be placed.",
            label_msgid='googlecoop_label_adsposition',
            description_msgid='googlecoop_help_adsposition',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        vocabulary='getAdsVocabulary',
        default='FORID:9',
        required=True,
        write_permission="Manage Portal"
    ),

    StringField(
        name='domain',
        default="www.google.com",
        widget=StringWidget(
            label="Google Domain",
            description="You can choose a different Google domain than google.com (such as google.ru or google.es). This makes your search results favor results from that country.",
            label_msgid='googlecoop_label_domain',
            description_msgid='googlecoop_help_domain',
            i18n_domain='googlecoop',
        ),
        required=True,
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='addStatus',
        widget=SelectionWidget(
            label="Advertising status",
            description="Specify whether your search engine is for a non-profit, university, or government website that should not have advertising on the results pages.",
            label_msgid='googlecoop_label_addStatus',
            description_msgid='googlecoop_help_addStatus',
            i18n_domain='googlecoop',
        ),
        multiValued=False,
        vocabulary='getAddStatusVocabulary',
        default="false",
        required=True,
        write_permission="Manage Portal",
        read_permission="View"
    ),

    BooleanField(
        name='labelKeywords',
        default="False",
        widget=BooleanField._properties['widget'](
            label="Use Keywords",
            description="Use your sites keywords as labels for the CSE",
            label_msgid='googlecoop_label_labelKeywords',
            description_msgid='googlecoop_help_labelKeywords',
            i18n_domain='googlecoop',
        ),
        required=True,
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='borderColor',
        default=336699,
        widget=StringWidget(
            label="Border",
            description="Color for the border around the search results",
            label_msgid='googlecoop_label_borderColor',
            description_msgid='googlecoop_help_borderColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='titleColor',
        default="0000CC",
        widget=StringWidget(
            label="Title",
            description="Color for the search result Title",
            label_msgid='googlecoop_label_titleColor',
            description_msgid='googlecoop_help_titleColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='backgroundColor',
        default="FFFFFF",
        widget=StringWidget(
            label="Background",
            description="Background Color",
            label_msgid='googlecoop_label_backgroundColor',
            description_msgid='googlecoop_help_backgroundColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='textColor',
        default=0,
        widget=StringWidget(
            label="Text",
            description="Color of the result description",
            label_msgid='googlecoop_label_textColor',
            description_msgid='googlecoop_help_textColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='linkColor',
        default=8000,
        widget=StringWidget(
            label="Links",
            description="Color for the URL",
            label_msgid='googlecoop_label_linkColor',
            description_msgid='googlecoop_help_linkColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='visitedColor',
        default=663399,
        widget=StringWidget(
            label="Visited Links",
            description="Color for visited URLs",
            label_msgid='googlecoop_label_visitedColor',
            description_msgid='googlecoop_help_visitedColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

    StringField(
        name='cachedColor',
        default=0,
        widget=StringWidget(
            label="Cached Link",
            description="Color for cached links",
            label_msgid='googlecoop_label_cachedColor',
            description_msgid='googlecoop_help_cachedColor',
            i18n_domain='googlecoop',
        ),
        read_permission="View",
        write_permission="Manage Portal"
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

googlecoop_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class googlecoop(UniqueObject, BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + \
        (getattr(BaseContent,'__implements__',()),)

    meta_type = 'googlecoop'
    suppl_views = ()
    typeDescription = "Google Coop"
    typeDescMsgId = 'description_edit_googlecoop'
    toolicon = 'google-coop.png'

    _at_rename_after_creation = True

    schema = googlecoop_schema

    ##code-section class-header #fill in your manual code here
    def getAdsVocabulary(self):
    	return ADPLACEMENT.items()

    def getAddStatusVocabulary(self):
        return ADDSTATUS.items()
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_googlecoop')
        self.setTitle('Google Coop')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer


    # tool should not appear in portal_catalog
    def at_post_edit_script(self):
        self.unindexObject()
        
        ##code-section post-edit-method-footer #fill in your manual code here
        ##/code-section post-edit-method-footer


    # Methods

    security.declarePublic('getGoCoopJS')
    def getGoCoopJS(self):
        """
        Return JS code to display your search results
        """
	JSCode ='''
  	var googleSearchIframeName = "results_%s";
  	var googleSearchFormName = "searchbox_%s";
  	var googleSearchFrameWidth = 600;
  	var googleSearchFrameborder = 0;
  	var googleSearchDomain = "%s";
  	var googleSearchPath = "/cse";
	'''  % (self.identifier, self.identifier, self.domain)

	return JSCode

    security.declarePublic('getAllRemoteURLs')
    def getAllRemoteURLs(self):
        """
        Returns all remote_urls from the portal catalog
        """
        URLs = self.portal_catalog.uniqueValuesFor('getRemoteUrl')
        AllURLs = '\n'.join(URLs)
        return AllURLs

    security.declarePublic('uniqueURLID')
    def uniqueURLID(self,URL=None):
        """
        Return an uniqe ID for a given URL
        """
        try:
            sig = md5.new(URL)
        except:
            sig = md5.new(self.identifier)
        urlId = "_cse_" + sig.hexdigest()
        return urlId

    security.declarePublic('getCseCrefJS')
    def getCseCrefJS(self):
        """
        Return JavaScript for a Linked Custom Search Engine
        """
        JSCode ='''
          var googleSearchIframeName = "results_cref";
          var googleSearchFormName = "searchbox_cref";
          var googleSearchFrameWidth = 600;
          var googleSearchFrameborder = 0;
          var googleSearchDomain = "%s";
          var googleSearchPath = "/cse";
        '''  % (self.domain)
        return JSCode

    security.declarePublic('subject2label')
    def subject2label(self,Keyword=None):
        """
        Convert a plone subject into a google label.
        Labels seem to have to constist solely lowercase letters and _ underscores
        """
        if Keyword:
           plone_tool = getToolByName(self, 'plone_utils', None)
           if plone_tool is None or not shasattr(plone_tool, 'normalizeString'):
                return KeyWord
           else:
                label=plone_tool.normalizeString(Keyword)
                label=label.replace('-','_')
                return label

    # Manually created methods

    def getAdsVocabulary(self):
    	return ADPLACEMENT.items()

    def getAddStatusVocabulary(self):
        return ADDSTATUS.items()
    ##/code-section class-header


registerType(googlecoop, PROJECTNAME)
# end of class googlecoop

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: googlecoop.py
#
# Copyright (c) 2007 by []
# Generator: ArchGenXML Version 1.6.0-beta-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Christian Ledermann <unknown>"""
__docformat__ = 'plaintext'


# Product configuration.
#
# The contents of this module will be imported into __init__.py, the
# workflow configuration and every content type module.
#
# If you wish to perform custom configuration, you may put a file
# AppConfig.py in your product's root directory. The items in there
# will be included (by importing) in this file if found.

from Products.CMFCore.permissions import setDefaultRoles

##code-section config-head #fill in your manual code here
from Products.Archetypes.public import DisplayList
##/code-section config-head


PROJECTNAME = "googlecoop"

# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Manage Portal"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner'))

product_globals = globals()

# Dependencies of Products to be installed by quick-installer
# override in custom configuration
DEPENDENCIES = []

# Dependend products - not quick-installed - used in testcase
# override in custom configuration
PRODUCT_DEPENDENCIES = []

# You can overwrite these two in an AppConfig.py:
# STYLESHEETS = [
#     {'id': 'my_global_stylesheet.css'},
#     {'id': 'my_contenttype.css',
#      'expression': 'python:object.getTypeInfo().getId() == "MyType"'}]
# You can do the same with JAVASCRIPTS.
STYLESHEETS = []
JAVASCRIPTS = []

##code-section config-bottom #fill in your manual code here
ADPLACEMENT = DisplayList((
	('FORID:9', 'Right'),
	('FORID:10', 'Top and Right'),
	('FORID:11', 'Top and Bottom'),
	))

ADDSTATUS = DisplayList((
        ('false', 'Show ads on results pages'),
        ('true', 'Do not show ads on results pages (for non-profits, universities, and government agencies only)'),
        ))
##/code-section config-bottom


# Load custom configuration not managed by archgenxml
try:
    from Products.googlecoop.AppConfig import *
except ImportError:
    pass



from setuptools import setup, find_packages
import os

version = '1.0'

setup(name='Products.googlecoop',
      version=version,
      description="Google Coop for Plone makes it easy for you to integrate a Google Co-op Custom Search engine in your plone site.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      maintainer='Vitaliy Podoba',
      maintainer_email='vitaliypodoba@gmail.com',
      url='http://plone.org/products/google-co-op',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
#
# File: Install.py
#
# Copyright (c) 2007 by []
# Generator: ArchGenXML Version 1.6.0-beta-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Christian Ledermann <unknown>"""
__docformat__ = 'plaintext'


import os.path
import sys
from StringIO import StringIO
from sets import Set
from App.Common import package_home
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import manage_addTool
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from zExceptions import NotFound, BadRequest

from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.config import TOOL_NAME as ARCHETYPETOOLNAME
from Products.Archetypes.atapi import listTypes
from Products.googlecoop.config import PROJECTNAME
from Products.googlecoop.config import product_globals as GLOBALS


def install(self, reinstall=False):
    """ External Method to install googlecoop """
    out = StringIO()
    print >> out, "Installation log of %s:" % PROJECTNAME

    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    try:
        from Products.googlecoop.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        import transaction
        transaction.savepoint(optimistic=True)

    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    # autoinstall tools
    portal = getToolByName(self,'portal_url').getPortalObject()
    for t in ['googlecoop']:
        try:
            portal.manage_addProduct[PROJECTNAME].manage_addTool(t)
        except BadRequest:
            # if an instance with the same name already exists this error will
            # be swallowed. Zope raises in an unelegant manner a 'Bad Request' error
            pass
        except:
            e = sys.exc_info()
            if e[0] != 'Bad Request':
                raise

    # hide tools in the search form
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        siteProperties = getattr(portalProperties, 'site_properties', None)
        if siteProperties is not None and siteProperties.hasProperty('types_not_searched'):
            for tool in ['googlecoop']:
                current = list(siteProperties.getProperty('types_not_searched'))
                if tool not in current:
                    current.append(tool)
                    siteProperties.manage_changeProperties(**{'types_not_searched' : current})

    # uncatalog tools
    for toolname in ['portal_googlecoop']:
        try:
            portal[toolname].unindexObject()
        except:
            pass

    # hide tools in the navigation
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties is not None and navtreeProperties.hasProperty('idsNotToList'):
            for toolname in ['portal_googlecoop']:
                current = list(navtreeProperties.getProperty('idsNotToList'))
                if toolname not in current:
                    current.append(toolname)
                    navtreeProperties.manage_changeProperties(**{'idsNotToList' : current})

    # register tools as configlets
    portal_controlpanel = getToolByName(self,'portal_controlpanel')
    portal_controlpanel.unregisterConfiglet('googlecoop')
    portal_controlpanel.registerConfiglet(
        'googlecoop', #id of your Tool
        'Google Coop', # Title of your Product
        'string:${portal_url}/portal_googlecoop/view',
        'python:True', # a condition
        'Manage Portal', # access permission
        'Products', # section to which the configlet should be added: (Plone, Products (default) or Member)
        1, # visibility
        'googlecoopID',
        'google-coop.png', # icon in control_panel
        'Google Co-op',
        None,
    )



    # enable portal_factory for given types
    factory_tool = getToolByName(self,'portal_factory')
    factory_types=[
        "googlecoop",
        ] + factory_tool.getFactoryTypes().keys()
    factory_tool.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)

    from Products.googlecoop.config import STYLESHEETS
    try:
        portal_css = getToolByName(portal, 'portal_css')
        for stylesheet in STYLESHEETS:
            try:
                portal_css.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': '',
            'media': 'all',
            'enabled': True}
            defaults.update(stylesheet)
            portal_css.registerStylesheet(**defaults)
    except:
        # No portal_css registry
        pass
    from Products.googlecoop.config import JAVASCRIPTS
    try:
        portal_javascripts = getToolByName(portal, 'portal_javascripts')
        for javascript in JAVASCRIPTS:
            try:
                portal_javascripts.unregisterResource(javascript['id'])
            except:
                pass
            defaults = {'id': ''}
            defaults.update(javascript)
            portal_javascripts.registerScript(**defaults)
    except:
        # No portal_javascripts registry
        pass

    # try to call a custom install method
    # in 'AppInstall.py' method 'install'
    try:
        install = ExternalMethod('temp', 'temp',
                                 PROJECTNAME+'.AppInstall', 'install')
    except NotFound:
        install = None

    if install:
        print >>out,'Custom Install:'
        try:
            res = install(self, reinstall)
        except TypeError:
            res = install(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom install'
    setup_tool = getToolByName(portal, 'portal_setup')
    # old_context = setup_tool.getImportContextID()
    # setup_tool.setImportContext('profile-googlecoop:default')
    # setup_tool.runAllImportSteps()
    # setup_tool.setImportContext(old_context)
    setup_tool.runAllImportStepsFromProfile(
        'profile-Products.googlecoop:default')
    print >> out, "Ran all GS import steps."
    return out.getvalue()


def uninstall(self, reinstall=False):
    out = StringIO()
    # unhide tools in the search form
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        siteProperties = getattr(portalProperties, 'site_properties', None)
        if siteProperties is not None and siteProperties.hasProperty('types_not_searched'):
            for tool in ['googlecoop']:
                current = list(siteProperties.getProperty('types_not_searched'))
                if tool in current:
                    current.remove(tool)
                    siteProperties.manage_changeProperties(**{'types_not_searched' : current})

    # unhide tools
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties is not None and navtreeProperties.hasProperty('idsNotToList'):
            for toolname in ['portal_googlecoop']:
                current = list(navtreeProperties.getProperty('idsNotToList'))
                if toolname in current:
                    current.remove(toolname)
                    navtreeProperties.manage_changeProperties(**{'idsNotToList' : current})


    # unregister tools as configlets
    portal_control_panel = getToolByName(self,'portal_controlpanel', None)
    if portal_control_panel is not None:
        portal_control_panel.unregisterConfiglet('googlecoop')
    # try to call a custom uninstall method
    # in 'AppInstall.py' method 'uninstall'
    try:
        uninstall = ExternalMethod('temp', 'temp',
                                   PROJECTNAME+'.AppInstall', 'uninstall')
    except:
        uninstall = None
    if uninstall:
        print >>out,'Custom Uninstall:'
        try:
            res = uninstall(self, reinstall)
        except TypeError:
            res = uninstall(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom uninstall'
    return out.getvalue()


def beforeUninstall(self, reinstall, product, cascade):
    """ try to call a custom beforeUninstall method in 'AppInstall.py'
        method 'beforeUninstall'
    """
    out = StringIO()
    try:
        beforeuninstall = ExternalMethod(
            'temp', 'temp',
            PROJECTNAME+'.AppInstall', 'beforeUninstall')
    except:
        beforeuninstall = []

    if beforeuninstall:
        print >>out, 'Custom beforeUninstall:'
        res = beforeuninstall(self,
                              reinstall=reinstall,
                              product=product,
                              cascade=cascade)
        if res:
            print >>out, res
        else:
            print >>out, 'no output'
    else:
        print >>out, 'no custom beforeUninstall'
    return (out,cascade)


def afterInstall(self, reinstall, product):
    """ try to call a custom afterInstall method in 'AppInstall.py' method
        'afterInstall'
    """
    out = StringIO()
    try:
        afterinstall = ExternalMethod('temp', 'temp',
                                   PROJECTNAME+'.AppInstall', 'afterInstall')
    except:
        afterinstall = None

    if afterinstall:
        print >>out, 'Custom afterInstall:'
        res = afterinstall(self,
                           product=None,
                           reinstall=None)
        if res:
            print >>out, res
        else:
            print >>out, 'no output'
    else:
        print >>out, 'no custom afterInstall'
    return out
